// written by Zheng Gengbin
//

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#else
#include <strings.h>
#endif

#ifdef HAVE_MALLOC_H
#include <malloc.h>
#endif

#include "compilerApi.h"
#include "strategy.h"
#include "anls_att.h"

extern void analyse_processor(SgStatement *);
extern void analyse_template(SgStatement *);
extern void analyse_dynamic(SgStatement *);
extern void analyse_inherit(SgStatement *);
extern void analyse_outofcore(SgStatement *);
extern void analyse_memory(SgStatement *);
extern void analyse_incoresize(SgStatement *);
extern void analyse_major(SgStatement *);
extern void analyse_shadow(SgStatement *);

// mainly analyse specification statements
// and perform some information collection on execution statements
void specification_analyse(SgStatement *s)
{
  switch(s->variant()) {

  case GLOBAL:
    {
      int org_numberOfChildren=s->numberOfChildrenList1();
      SgStatement * * children = new SgStatement * [org_numberOfChildren];

      int i;

      for (i=0; i<org_numberOfChildren; i++)
	children[i]=s->childList1(i);
      for (i=0; i<org_numberOfChildren; i++)
	specification_analyse(children[i]);
      
      delete [] children;

      break;
    };

  case PROG_HEDR:
    {
      int org_numberOfChildren=s->numberOfChildrenList1();
      SgStatement * * children = new SgStatement * [org_numberOfChildren];

      int i; 

      for (i=0; i<org_numberOfChildren; i++)
	children[i]=s->childList1(i);

      // do nothing?;
      in_main_program=on;

      for (i=0; i<org_numberOfChildren; i++)
	specification_analyse(children[i]);
      
      delete [] children;

      in_main_program=off;

      break;

    };

  case PROC_HEDR:
    {
      SgProcHedrStmt * prochedr = (SgProcHedrStmt *)s;
      int org_numberOfChildren=s->numberOfChildrenList1();
      SgStatement * * children = new SgStatement * [org_numberOfChildren];

      int i;

      for (i=0; i<org_numberOfChildren; i++)
	children[i]=s->childList1(i);

      in_procedure=on;

      for (i=0; i<prochedr->numberOfParameters(); i++) {
	prochedr->parameter(i)->addAttribute(DUMP_Def, NULL, 0);
	prochedr->addAttribute(DUMP_Def, NULL, 0);
      }

      for (i=0; i<org_numberOfChildren; i++)
	specification_analyse(children[i]);
      
      delete [] children;
      
      in_procedure = off;

      break;
    };

  case FUNC_HEDR:
    {
      SgProcHedrStmt * prochedr = (SgProcHedrStmt *)s;
      int org_numberOfChildren=s->numberOfChildrenList1();
      SgStatement * * children = new SgStatement * [org_numberOfChildren];

      int i;

      for (i=0; i<org_numberOfChildren; i++)
	children[i]=s->childList1(i);

      in_procedure=on;

      for (i=0; i<prochedr->numberOfParameters(); i++) {
	prochedr->parameter(i)->addAttribute(DUMP_Def, NULL, 0);
	prochedr->addAttribute(DUMP_Def, NULL, 0);
      }

      for (i=0; i<org_numberOfChildren; i++)
	specification_analyse(children[i]);
      
      delete [] children;
      
      in_procedure = off;

      break;
    };

  case INTERFACE_STMT:
    {
      SgProcHedrStmt * prochedr = (SgProcHedrStmt *)s;
      int org_numberOfChildren=s->numberOfChildrenList1();
      SgStatement * * children = new SgStatement * [org_numberOfChildren];

      int i;

      for (i=0; i<org_numberOfChildren; i++)
	children[i]=s->childList1(i);

      in_interface=on;

      for (i=0; i<org_numberOfChildren; i++)
	specification_analyse(children[i]);
      
      delete [] children;
      
      in_interface = off;

      break;
    };

  case PROCESSORS_STMT:
    analyse_processor(s);
    break;

  case TEMPLATE_STMT:
    analyse_template(s);
    break;

  case DYNAMIC_DECL:
    analyse_dynamic(s);
    break;

  case ALIGN_DECL:
//    analyse_alignment(s);
    break;
    
  case DISTRIBUTE_DECL:
//    analyse_distribute(s);
    break;

  case INHERIT_DECL:
    analyse_inherit(s);
    break;

  case OUTOFCORE_DECL:
    analyse_outofcore(s);
    break;

  case MEMORY_DECL:
    analyse_memory(s);
    break;

  case INCORESIZE_DECL:
    analyse_incoresize(s);
    break;

  case MAJOR_DECL:
    analyse_major(s);
    break;

  case SHADOW_STMT:
    analyse_shadow(s);
    break;

  case VAR_DECL:
    break;

  case FOR_NODE:
    {
//      do_level++;

      int org_numberOfChildren=s->numberOfChildrenList1();

      SgStatement * * children = new SgStatement * [org_numberOfChildren];

      int i;

      for (i=0; i<org_numberOfChildren; i++)
	children[i]=s->childList1(i);

      for (i=0; i<org_numberOfChildren; i++)
	specification_analyse(children[i]);
      
      delete [] children;
      
//      do_level--;

      break;
    }

  case LOGIF_NODE:
  case LOGIF_STMT:
    {
//      specification_analyse_if_stmt(s);	// by zheng.gb

      int org_numberOfChildren1=s->numberOfChildrenList1();

      SgStatement * * children1 = new SgStatement * [org_numberOfChildren1];

      int org_numberOfChildren2=s->numberOfChildrenList2();

      SgStatement * * children2 = new SgStatement * [org_numberOfChildren2];

      int i;

      for (i=0; i<org_numberOfChildren1; i++)
	children1[i]=s->childList1(i);

      for (i=0; i<org_numberOfChildren1; i++)
	specification_analyse(children1[i]);
      
      delete [] children1;

      for (i=0; i<org_numberOfChildren2; i++)
	children2[i]=s->childList2(i);

      for (i=0; i<org_numberOfChildren2; i++)
	specification_analyse(children2[i]);
      
      delete [] children2;

      break;
    }
  case ELSEIF_NODE:
  case SWITCH_NODE:
  case CASE_NODE:
    {
      int org_numberOfChildren1=s->numberOfChildrenList1();

      SgStatement * * children1 = new SgStatement * [org_numberOfChildren1];

      int org_numberOfChildren2=s->numberOfChildrenList2();

      SgStatement * * children2 = new SgStatement * [org_numberOfChildren2];

      int i;

      for (i=0; i<org_numberOfChildren1; i++)
	children1[i]=s->childList1(i);

      for (i=0; i<org_numberOfChildren1; i++)
	specification_analyse(children1[i]);
      
      delete [] children1;

      for (i=0; i<org_numberOfChildren2; i++)
	children2[i]=s->childList2(i);

      for (i=0; i<org_numberOfChildren2; i++)
	specification_analyse(children2[i]);
      
      delete [] children2;

      break;

    }

  case ASSGOTO_NODE:
  case STOP_STAT:
    // do nothing;
    break;


  case ASSIGN_STAT:
/*
    intrinsic2sub(s);
    if (s->variant() == ASSIGN_STAT ) {
      specification_analyse_assign(s);
      break;
    }
*/
    // else continue process 
    break;

  case PROC_STAT:
    //do nothing at presenttime. we will specification_analyse INTERFACE later;
//    specification_analyse_proc_stat(s);
    break;

  case FORALL_STMT:
    {       

      break;
    }

  case OPEN_STAT:
  case CLOSE_STAT:
// do nothing
    break;

  case READ_STAT:
  case PRINT_STAT:
  case WRITE_STAT:
//    analyse_datatransfer(s);
    break;

  case CONTROL_END:
  case PARAM_DECL:
  case ENDDO_STMT:
  case  FORMAT_STAT:
   // statement need do nothing;
    break;

  case RETURN_STAT: {
/*
    SgStatement *scope = s->getScopeForDeclare();
    scope->addAttribute(RETURN_LABEL, NULL, 0);
*/
    break;
   }

  case EXIT_STMT:
  case IMPL_DECL:
  case USE_STMT:
   // statement need do nothing;
   break;

  default:
//    printf("Not touched statement in specification_analyse on line %d,variant:%d\n", s->lineNumber(),s->variant());
    break;
  };

}


